import json
import logging

from django_swappable_tasks.handlers.celery import CeleryHandler
from django_swappable_tasks.handlers.google_cloud_tasks import GoogleCloudTasks

logger = logging.getLogger(__name__)


def get_fully_qualified_task_name(obj):
    """
    :param obj: a valid python object
    :return: fully qualified dotted name
    For example from betarcade.apps.bets.tasks import update_fixtures_from_remote
    will return betarcade.apps.bets.tasks.update_fixtures_from_remote
    """
    return ".".join([obj.__module__, obj.__name__])


def import_method_from_name(function_string):
    import importlib
    mod_name, func_name = function_string.rsplit('.', 1)
    mod = importlib.import_module(mod_name)
    return getattr(mod, func_name)


def split_comma_separated_args(comma_separated_args):
    if comma_separated_args is None:
        return []
    return [int(i) if i.isdigit() else str(i) for i in comma_separated_args.split(',')]


def dump_args_into_comma_separated_list(task_args):
    if not isinstance(task_args, list):
        task_args_list = list()
        task_args_list.append(task_args)
        task_args = task_args_list
    comma_separated_args = ','.join(str(arg) for arg in task_args)
    return comma_separated_args


def run_task(task_path, args_json, kwargs_json):
    args, kwargs = [], {}
    if args_json is not None:
        args = split_comma_separated_args(args_json)
    if kwargs_json is not None:
        kwargs = json.loads(kwargs_json)
    try:
        if task_path is not None:
            task = import_method_from_name(task_path)
            logger.debug("Running {}".format(task_path))
            result = task(*args, **kwargs)
            logger.debug("Completed {}. Result : {}.".format(task_path, str(result)))
            return True, str(result)
        else:
            return False, "Task not provided."
    except Exception as e:
        logger.error(e)
        return False, "Error : {}".format(str(e))


def process_task_asynchronously(task, queue, task_args=[], task_kwargs={}, *args, **kwargs):
    from django.conf import settings
    if settings.DEFAULT_ASYNC_TASKS_HANDLER == "CELERY":
        return CeleryHandler.add_task_to_queue(task, task_args, task_kwargs, queue)
    elif settings.DEFAULT_ASYNC_TASKS_HANDLER == "GOOGLE_CLOUD_TASKS":
        return GoogleCloudTasks.add_task_to_queue(task, task_args, task_kwargs, queue)
    else:
        raise ValueError("Unknown tasks handler {}.".format(settings.DEFAULT_ASYNC_TASKS_HANDLER))



import logging

from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from django_swappable_tasks.utils import run_task

logger = logging.getLogger(__name__)


@method_decorator(csrf_exempt, name='dispatch')
class TasksHandlerView(View):
    def post(self, request, *args, **kwargs):
        task = request.GET.get('task', None)
        args_json = request.GET.get('args', None)
        kwargs_json = request.GET.get('kwargs', None)

        success, result = run_task(task_path=task, args_json=args_json, kwargs_json=kwargs_json)
        return HttpResponse("Status {} Result : {}".format(success, result))

    def get(self, request, *args, **kwargs):
        task = request.GET.get('task', None)
        args_json = request.GET.get('args', None)
        kwargs_json = request.GET.get('kwargs', None)

        success, result = run_task(task_path=task, args_json=args_json, kwargs_json=kwargs_json)
        return HttpResponse("Status {} Result : {}".format(success, result))



from django.urls import path

from django_swappable_tasks.views import TasksHandlerView

app_name = "django_swappable_tasks"

urlpatterns = [
    path('tasks/', TasksHandlerView.as_view(), name='tasks')
]



from distutils.core import setup

setup(
    name='django_swappable_tasks',  # How you named your package folder (MyLib)
    packages=['django_swappable_tasks'],  # Chose the same as "name"
    version='0.1',  # Start with a small number and increase it with every change you make
    license='gpl-3.0',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='Switch between asynchronous task handlers in your django without in seconds.',
    # Give a short description about your library
    author='Jerry Shikanga',  # Type in your name
    author_email='jerryshikanga@gmail.com',  # Type in your E-Mail
    url='https://github.com/jerryshikanga/django_swappable_tasks',
    # Provide either the link to your github or to your website
    download_url='https://github.com/jerryshikanga/django_swappable_tasks/archive/v_01.tar.gz',  # I explain this later on
    keywords=['Django', 'Tasks', 'Asynchronous', 'Handlers', 'Swap', 'Google Cloud Tasks'],
    # Keywords that define your package best
    install_requires=[  # I get to this in a second
        'google-cloud-tasks',
        'celery',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
